class DiscordUser:
    def __init__(self, **kwargs) -> None:
        self.id = kwargs.get("id")
        self.username = kwargs.get("username")
        self.discriminator = kwargs.get("discriminator")
        self.avatar = kwargs.get("avatar")
        self.bot = kwargs.get("bot", False)
        self.image_url = kwargs.get("imageUrl", "https://cdn.discordapp.com/embed/avatars/0.png")

    def __str__(self) -> str:
        return f"{self.username}#{self.discriminator}"



from requests import Response

from .console import console, MASZLogLevel
from .request_adapter import MASZRequestAdapter
from .exceptions import MASZBaseException, MASZLoginFailure
from .obj import DiscordUser



class MASZClient:
    def __init__(self, url: str, token: str, api_version: int = 1, log_level: MASZLogLevel = MASZLogLevel.INFO) -> None:
        self.url = url
        self.token = token
        self.api_version = api_version
        self.request_adapter = MASZRequestAdapter(self.url, self.token, self.api_version)
        console.log_level = log_level

        try:
            with console.info_status(f"[bold green]Connecting to MASZ at {url} ...") as status:
                service_health = self.request_adapter.get_current_health()
                if service_health.status.lower() == 'ok':
                    if service_health.response_time < 200:
                        console.info(f":white_check_mark: Ping [bright_green]'{service_health.status.upper()}' {service_health.response_time}[/bright_green] ms.")
                    else:
                        console.info(f":white_check_mark: Ping [bright_green]'{service_health.status.upper()}'[/bright_green][bright_yellow] {service_health.response_time}[/bright_yellow] ms.")
                else:
                    console.info(f":white_check_mark: Ping [bright_red]'{service_health.status.upper()}' {service_health.response_time}[/bright_red] ms.")

        except MASZBaseException as e:
            console.critical(f":exclamation: [red]MASZ API at {url} seems to be unhealthy or unreachable.")
            exit(1)
        
        try:
            with console.info_status(f"[bold bright_green]Logging in...") as status:
                self.me = self.request_adapter.get_current_user()
                console.info(f":white_check_mark: [bright_green]Logged in[/bright_green] to MASZ at {self.url} as [bright_blue]{self.me}[/bright_blue].")

        except MASZBaseException as e:
            console.critical(f":exclamation: [red]Failed to login.")
            exit(1)



from requests import Response
import requests
from requests.exceptions import URLRequired

class MASZRequestHandler:
    def __init__(self, url: str, token: str, api_version: int, header: str, header_prefix: str) -> None:
        self.url = url.rstrip('/')
        self.token = token
        self.api_version = api_version
        self.header = header
        self.header_prefix = header_prefix
        self.headers = {self.header: f"{self.header_prefix}{self.token}"}

    def get(self, resource: str, params: dict = dict(), headers: dict = dict()) -> Response:
        final_headers = {**headers, **self.headers}
        return requests.get(f"{self.url}/api/v{self.api_version}/{resource.lstrip('/')}", headers=final_headers, params=params)



import time
from requests import Response

from .console import console
from .masz_request import MASZRequestHandler
from .exceptions import *
from .obj import *

class MASZRequestAdapter:
    def __init__(self, url: str, token: str, api_version: int = 1, header: str = "Authorization", header_prefix: str = "Bearer ") -> None:
        self.request_handler = MASZRequestHandler(url, token, api_version, header, header_prefix)

    def _get(self, resource: str, params: dict = dict(), headers: dict = dict()) -> Response:
        try:
            r = self.request_handler.get(resource, params, headers)
            if r.status_code not in [200, 201]:
                console.verbose(f"[bright_red]{r.status_code}[bright_magenta] /{resource.lstrip('/')}")
            else:
                console.verbose(f"[bright_green]{r.status_code}[bright_magenta] /{resource.lstrip('/')}")
        except Exception as e:
            raise MASZRequestFailure("Request failed", e)
        if r.status_code == 401:
            raise MASZLoginFailure("Unauthorized", r.text)
        if r.status_code not in [200, 201]:
            raise MASZRequestFailure(f"Invalid response code {r.status_code}", r.text)
        return r

    def get_current_user(self) -> AppUser:
        r = self._get("/discord/users/@me")
        return AppUser(**r.json())


    def get_current_health(self) -> Status:
        start = time.clock()
        r = self._get("/health", dict(), {'Accept': 'application/json'})
        request_time = time.clock() - start
        return Status(round(request_time*1000, 2) , **r.json())



from dateutil.parser import parser as date_parser

class Status:
    def __init__(self, response_time: int, **kwargs) -> None:
        self.status = kwargs.get("status", '')
        self.name = kwargs.get("name")
        self.server_time = date_parser(kwargs.get("server_time"))
        self.server_time_utc = date_parser(kwargs.get("server_time_utc"))
        self.response_time = response_time

    def __str__(self) -> str:
        return f"{self.status.upper()} ({self.response_time}ms)"



from setuptools import setup, find_packages

with open("readme.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
  name='masz',
  py_modules=["masz"],
  version='1.0.2',
  license='MIT',
  description='API Wrapper for MASZ (A discord moderation bot)',
  author='zaanposni',
  author_email='me@zaanposni.com',
  url='https://github.com/zaanposni/masz-api-wrapper',
  keywords=['MASZ', 'API', 'DISCORD', 'WRAPPER', 'JSON', 'REST', 'MODERATION', 'TOKEN', 'PYTHON'],
  packages=find_packages(exclude=["*tests"]),
  package_data={
    "masz": ["masz/*"]
  },
  install_requires=[
          'requests',
          'typing',
          'python-dateutil',
          'rich'
      ],
  classifiers=[
    'Development Status :: 1 - Planning',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
  ],
  long_description=long_description,
  long_description_content_type="text/markdown"
)
